-- Copyright 2012 flashkot
-- Copyright 2014 Philipp Überbacher (port to Löve 0.9.1)

-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at

-- http://www.apache.org/licenses/LICENSE-2.0

-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.

time = 0
lineNb = 512
effect = love.graphics.newShader[[
	extern number time;

	// noise effect intensity value (0 = no effect, 1 = full effect)
	extern number nIntensity;

	// scanlines effect intensity value (0 = no effect, 1 = full effect)
	extern number sIntensity;

	// scanlines effect count value (0 = no effect, 4096 = full effect)
	extern number sCount;

	vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc)
	{
		vec4 cTextureScreen = Texel( tex, tc);
		
		number x = tc.x * tc.y * time * 1000.0;
		x = mod( x, 13.0 ) * mod( x, 123.0 );
		number dx = mod( x, 0.01 );
		
		vec3 cResult = cTextureScreen.rgb + cTextureScreen.rgb * clamp( 0.1 + dx * 100.0, 0.0, 1.0 );
		
		vec2 sc = vec2( sin( tc.y * sCount ), cos( tc.y * sCount ) );
		
		cResult += cTextureScreen.rgb * vec3( sc.x, sc.y, sc.x ) * sIntensity;
		
		cResult = cTextureScreen.rgb + clamp( nIntensity, 0.0,1.0 ) * ( cResult - cTextureScreen.rgb );
		
			//cResult = vec3( 0, (1-cResult.r) * 0.3 + cResult.g * 0.59 + cResult.b * 0.11,0 ); //grayscale
		cResult = vec3( cResult.r * 0.5 , cResult.g * 0.9 , cResult.b * 0.5); //grayscale
		return vec4( cResult, cTextureScreen.a );
		
	}
]]



local img1, img2
local first_image = true
local wait_image = false
local stop = false
local stopcount = 0
local loader_thread
local channel_filename
local channel_imagedata

local cur_frame = 0
local next_load = 3

local frame_width, frame_height = 180, 100
local scale_factor = 4
local frames_in_row, rows_num = 5, 5
local video_fps, video_spf = 25, 1/25

local frames_per_image = frames_in_row * rows_num

local quads = {}
local current_quad = 1
local frame_time = 0

local audio_position = 0

local sound

function create_quads (width, height, num_x, num_y)
	local i,j

	for j = 0, num_y - 1, 1 do
		for i = 0, num_x - 1, 1 do
			table.insert(quads, love.graphics.newQuad(width * i, height * j, width, height, width * num_x, height * num_y ))
		end
	end
end



function love.load()

	loader_thread = love.thread.newThread("loader_thread.lua")
	channel_filename = love.thread.getChannel("filename")
	channel_imagedata = love.thread.getChannel("imagedata")
	loader_thread:start()

	channel_filename:push("media/sec_001.jpg")
	imagedata = channel_imagedata:demand()

	img1 = love.graphics.newImage(  imagedata )

	channel_filename:push("media/sec_002.jpg")
	imagedata = channel_imagedata:demand()

	img2 = love.graphics.newImage(imagedata)

	create_quads(frame_width, frame_height, frames_in_row, rows_num)

	sound = love.audio.newSource("media/audio.ogg")

	love.window.setMode(frame_width * scale_factor, frame_height * scale_factor)

	love.audio.play(sound)
end

function love.draw()
	effect:send("time",time)
	effect:send("nIntensity", 0.9)
	effect:send("sIntensity", 0.6)
	effect:send("sCount", 2200)
	love.graphics.setShader(effect)

	if stop then
		stopcount = stopcount + 1
	end

	if stopcount <= frames_per_image then
		if first_image then
			love.graphics.draw(img1, quads[current_quad], 0, 0, 0, scale_factor, scale_factor)
		else
			love.graphics.draw(img2, quads[current_quad], 0, 0, 0, scale_factor, scale_factor)
		end
	end
end

function love.update(dt)
	time=time+dt

	dt = sound:tell() - audio_position
	audio_position = sound:tell()
	if sound:isStopped() then
		love.event.quit()
	end

	frame_time = frame_time + dt

	if not stop then
		if wait_image then
			imagedata = channel_imagedata:demand()
			if imagedata then
				if first_image then
					img2 = love.graphics.newImage(imagedata)
				else
					img1 = love.graphics.newImage(imagedata)
				end
				wait_image = false
			end
			-- ask for garbage collection once per file load
			collectgarbage("collect")
		end
	end

	if frame_time >= video_spf then
		frame_time = frame_time - video_spf
		current_quad = current_quad + 1
		if current_quad > frames_per_image then
			current_quad = 1
			channel_filename:push(string.format("media/sec_%03d.jpg", next_load))
			wait_image = true
			if not love.filesystem.exists(string.format("media/sec_%03d.jpg", next_load)) then
				stop = true
			end
			next_load = next_load + 1

			if first_image then
				first_image = false
			else
				first_image = true
			end
		end
	end
end

function love.keypressed(key)
	if key == "escape" then
		love.event.quit()
	end
end
